<?php
/**
 * Theme Widgets
 *
 * @package WordPress
 * @since Wild Book 1.1
 */

/**
 * About Me widget class
 *
 * @since Wild Book 1.1
 */
class Wild_Book_About_Me extends WP_Widget {

  private $social_networks;
  
  /**
   * Constructor.
   */
  public function __construct() {
    parent::__construct( 'wdb_widget_about_me', esc_html__( 'About Blog & Social Buttons', 'wildbook' ), array(
      'classname'   => 'wdb-widget-about-me',
      'description' => esc_html__( 'Use this widget to show information about your blog.', 'wildbook' ),
    ) );
    
    // Available social networks
    $this->social_networks = array(       
      'facebook'   => esc_html__( 'Facebook', 'wildbook' ),
      'twitter'    => esc_html__( 'Twitter', 'wildbook' ),
      'googleplus' => esc_html__( 'Google Plus', 'wildbook' ),
      'instagram'  => esc_html__( 'Instagram', 'wildbook' ),
      'tumblr'     => esc_html__( 'Tumblr', 'wildbook' ),
      'bloglovin'  => esc_html__( 'Bloglovin', 'wildbook' ),
      'pinterest'  => esc_html__( 'Pinterest', 'wildbook' ),
      'dribbble'   => esc_html__( 'Dribbble', 'wildbook' ),
      'linkedin'   => esc_html__( 'LinkedIn', 'wildbook' ),
      'behance'    => esc_html__( 'Behance', 'wildbook' ),
      'github'     => esc_html__( 'Github', 'wildbook' ),
      'medium'     => esc_html__( 'Medium', 'wildbook' ),
      'flickr'     => esc_html__( 'Flickr', 'wildbook' ),
      'vimeo'      => esc_html__( 'Vimeo', 'wildbook' ),
      'youtube'    => esc_html__( 'YouTube', 'wildbook' ),
      'f500px'     => esc_html__( '500px', 'wildbook' ),
      'vkontakte'  => esc_html__( 'Vkontakte', 'wildbook' ),
      'rss'        => esc_html__( 'Rss', 'wildbook' ),
      'email'      => esc_html__( 'Email', 'wildbook' )
    );

    add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );
  }
  
  /**
   * Enqueue admin widgets page scripts and styles.
   *
   * @access public
   */ 
  public function admin_enqueue_scripts( $hook ) {
    if ( 'widgets.php' != $hook )
          return;

    wp_enqueue_media();
    wp_enqueue_script( 'wildbook-widget-about-me-js',  get_template_directory_uri() . '/js/admin/widget-about-me.js', array( 'jquery' ), '1.0', true );
    wp_enqueue_style(  'wildbook-widget-about-me-css', get_template_directory_uri() . '/css/admin/widget-about-me.css', array(), '1.0');
    
    wp_localize_script( 
      'wildbook-widget-about-me-js', 
      'widlbookWidgetAboutMe', 
      array( 
        'selectText'       => esc_html__( 'Select Image', 'wildbook' ),
        'changeText'       => esc_html__( 'Change Image', 'wildbook' ),
        'removeText'       => esc_html__( 'Remove', 'wildbook' ),
        'mediaFrameTitle'  => esc_html__( 'Select Image', 'wildbook' ),
        'mediaFrameButton' => esc_html__( 'Choose Image', 'wildbook' )
      )
    );  
  }
  
  /**
   * Output the HTML for this widget.
   *
   * @access public
   *
   * @param array $args     An array of standard parameters for widgets in this theme.
   * @param array $instance An array of settings for this widget instance.
   */
  public function widget( $args, $instance ) {

    $title = apply_filters( 'widget_title', empty( $instance['title'] ) ? '' : $instance['title'], $instance, $this->id_base );
    $text = empty( $instance['text'] ) ? '' : $instance['text'];
    
    echo $args['before_widget'];
    if ( ! empty( $title ) ) {
      printf( '%s%s%s', $args['before_title'], $title, $args['after_title'] );
    } 
    
    $portrait_image = wp_get_attachment_image_src( $instance['portrait_image_id'], 'medium-thumbnail' );
    if( $portrait_image ) {
      printf( '<div class="portrait"><img src="%s" alt="" /></div>', esc_url( $portrait_image['0'] ) );
    }
    
    if( $instance['name'] ) {
      printf( '<h4>%s</h4>', $instance['name'] );
    }
      
    $more_link = '';
    
    if( $instance['more_link_url'] ) {
      $more_link_text = empty( $instance['more_link_text'] ) ? esc_html__( 'Continue reading &rarr;', 'wildbook' ) : $instance['more_link_text'];
      $more_link = sprintf( '<a href="%s">%s</a>', esc_url( $instance['more_link_url'] ), $more_link_text );
    }

    if( $text ) {
      printf( '<div class="textwidget">%s %s</div>', !empty( $instance['filter'] ) ? wpautop( $text ) : $text, $more_link );
    }
    
    $social_profiles = '';
    
    foreach( $this->social_networks as $network_id => $network_name ) {
      if( !empty( $instance[$network_id] ) ) {
        $social_profiles .= sprintf( '<a href="%1$s" class="social-profile %3$s" title="%2$s"><i class="fa fa-%3$s"></i></a>', 
          $network_id == 'email' ? ( is_email( $instance[$network_id] ) ? 'mailto:' . $instance[$network_id] : '' ) : esc_url( $instance[$network_id] ), 
          $network_name, 
          $network_id 
        );
      }
    }
    
    if( $social_profiles ) {
      printf( '<div class="social-profiles">%s</div>', $social_profiles );
    }
    
    echo $args['after_widget']; 
  }
  
  /**
   * Deal with the settings when they are saved by the admin.
   *
   * Here is where any validation should happen.
   *
   * @param array $new_instance New widget instance.
   * @param array $instance     Original widget instance.
   * @return array Updated widget instance.
   */
  function update( $new_instance, $instance ) {
    $instance['title'] = strip_tags( $new_instance['title'] );
    $instance['portrait_image_id'] = absint( $new_instance['portrait_image_id'] );
    $instance['name'] = strip_tags( $new_instance['name'] );
    
    if ( current_user_can('unfiltered_html') )
      $instance['text'] = $new_instance['text'];
    else
      $instance['text'] = stripslashes( wp_filter_post_kses( addslashes($new_instance['text']) ) ); // wp_filter_post_kses() expects slashed
      
    $instance['filter']         = isset( $new_instance['filter'] );
    $instance['more_link_url']  = esc_url( $new_instance['more_link_url'] );
    $instance['more_link_text'] = strip_tags( $new_instance['more_link_text'] );
    
    foreach( $this->social_networks as $key => $val ) {
      if( $key == 'email' ) {
        $instance['email'] = is_email( $new_instance['email'] ) ? $new_instance['email'] : '';
      } else {
        $instance[$key] = esc_url( $new_instance[$key] );
      }
    }

    return $instance;
  }
  
  /**
   * Display the form for this widget on the Widgets page of the Admin area.
   *
   * @param array $instance
   */
  function form( $instance ) {
    $instance = wp_parse_args( (array) $instance, array( 
      'title'             => '', 
      'portrait_image_id' => 0,
      'name'              => '', 
      'text'              => '', 
      'filter'            => false,
      'more_link_url'     => '',
      'more_link_text'    => esc_html__( 'Continue reading &rarr;', 'wildbook' ),
      //socials
      'facebook'       => '',
      'twitter'        => '',
      'googleplus'     => '',
      'instagram'      => '',
      'tumblr'         => '',
      'bloglovin'      => '',
      'pinterest'      => '',
      'dribbble'       => '',
      'linkedin'       => '',
      'behance'        => '',
      'github'         => '',
      'medium'         => '',
      'flickr'         => '',
      'vimeo'          => '',
      'youtube'        => '',
      'f500px'         => '',
      'vkontakte'      => '',
      'rss'            => '',
      'email'          => ''
    ) );  
  ?>
  
    <p><label for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"><?php esc_html_e( 'Title:', 'wildbook' ); ?></label>
    <input id="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>" class="widefat" name="<?php echo esc_attr( $this->get_field_name( 'title' ) ) ; ?>" type="text" value="<?php echo esc_attr( $instance['title'] ); ?>"></p>
    
    <div class="portrait-image">
      <label><?php esc_html_e( 'Photo:', 'wildbook' ); ?></label>
      
      <div class="selected-portrait">
        <?php 
          $portrait = wp_get_attachment_image_src( $instance['portrait_image_id'], 'thumbnail' );
          if( $portrait ) {
            printf( '<img src="%s" alt="" />', esc_url( $portrait['0'] ) );
          }
        ?>
      </div>
      
      <div class="portrait-actions">
        <?php 
          printf( '<button class="select-portrait-image button">%s</button> ', $portrait ? esc_html__( 'Change Image', 'wildbook' ) : esc_html__( 'Select Image', 'wildbook' ) );

          if( $portrait ) {
            printf( '<button class="remove-portrait-image button">%s</button>', esc_html__( 'Remove', 'wildbook' ) ); 
          }
        ?>
      </div>
      
      <input id="<?php echo esc_attr( $this->get_field_id( 'portrait_image_id' ) ); ?>" class="widefat portrait-id" name="<?php echo esc_attr( $this->get_field_name( 'portrait_image_id' ) ); ?>" type="hidden" value="<?php echo absint( $instance['portrait_image_id'] ); ?>">
    </div>
    
    <p><label for="<?php echo esc_attr( $this->get_field_id( 'name' ) ); ?>"><?php esc_html_e( 'Name:', 'wildbook' ); ?></label>
    <input id="<?php echo esc_attr( $this->get_field_id( 'name' ) ); ?>" class="widefat" name="<?php echo esc_attr( $this->get_field_name( 'name' ) ); ?>" type="text" value="<?php echo esc_attr( $instance['name'] ); ?>"></p>
    
    <p><label for="<?php echo esc_attr( $this->get_field_id( 'text' ) ); ?>"><?php esc_html_e( 'Text:', 'wildbook' ); ?></label>
    <textarea class="widefat" rows="5" cols="20" id="<?php echo esc_attr( $this->get_field_id('text') ); ?>" name="<?php echo esc_attr( $this->get_field_name('text') ); ?>"><?php echo esc_textarea( $instance['text'] ); ?></textarea>

    <p><input id="<?php echo esc_attr( $this->get_field_id('filter') ); ?>" name="<?php echo esc_attr( $this->get_field_name('filter') ); ?>" type="checkbox" <?php checked(isset($instance['filter']) ? $instance['filter'] : 0); ?> />&nbsp;<label for="<?php echo esc_attr( $this->get_field_id('filter') ); ?>"><?php esc_html_e( 'Automatically add paragraphs', 'wildbook' ); ?></label></p>
  
    <p><label for="<?php echo esc_attr( $this->get_field_id( 'more_link_url' ) ); ?>"><?php esc_html_e( 'Link URL:', 'wildbook' ); ?></label>
    <input id="<?php echo esc_attr( $this->get_field_id( 'more_link_url' ) ); ?>" class="widefat" name="<?php echo esc_attr( $this->get_field_name( 'more_link_url' ) ); ?>" type="text" value="<?php echo esc_url( $instance['more_link_url'] ); ?>"></p>
    
    <p><label for="<?php echo esc_attr( $this->get_field_id( 'more_link_text' ) ); ?>"><?php esc_html_e( 'Link text:', 'wildbook' ); ?></label>
    <input id="<?php echo esc_attr( $this->get_field_id( 'more_link_text' ) ); ?>" class="widefat" name="<?php echo esc_attr( $this->get_field_name( 'more_link_text' ) ); ?>" type="text" value="<?php echo esc_attr( $instance['more_link_text'] ); ?>"></p>

    <?php foreach( $this->social_networks as $network_id => $network_name ) : ?>
      <p><label for="<?php echo esc_attr( $this->get_field_id( $network_id ) ); ?>"><?php echo esc_attr( $network_name . ':' ); ?></label>
      <input id="<?php echo esc_attr( $this->get_field_id( $network_id ) ); ?>" class="widefat" name="<?php echo esc_attr( $this->get_field_name( $network_id ) ); ?>" type="text" value="<?php echo ( $network_id == 'email' ) ? esc_attr( $instance[$network_id] ) : esc_url( $instance[$network_id] ); ?>"></p> 
    <?php endforeach;
  }
}

/**
 * WP Instagram Widget
 *
 * A WordPress widget for showing your latest Instagram photos
 *
 * @source https://github.com/cftp/wp-instagram-widget
 * @author Scott Evans (Code For The People)
 * @license GPLv2 or later
 * @since Wild Book 1.3
 */
class Wild_Book_Instagram_Widget extends WP_Widget {
  /**
   * Constructor.
   */
  public function __construct() {
    parent::__construct( 'wdb-instagram-feed', esc_html__( 'Instagram', 'wildbook' ), array(
      'classname'   => 'wdb-instagram-feed',
      'description' => esc_html__( 'Displays your latest Instagram photos.', 'wildbook' ),
    ) );
  }

  function widget( $args, $instance ) {

    extract($args, EXTR_SKIP);

    $title = empty($instance['title']) ? '' : apply_filters( 'widget_title', $instance['title'] );
    $username = empty($instance['username']) ? '' : $instance['username'];
    $limit = empty($instance['number']) ? 9 : $instance['number'];
    $size = empty( $instance['size'] ) ? 'large' : $instance['size'];
    $link = empty($instance['link']) ? '' : $instance['link'];

    echo $args['before_widget'];
    
    if ( ! empty( $title ) ) {
      printf( '%s%s%s', $args['before_title'], $title, $args['after_title'] );
    } 

    if ( '' !== $username ) {

      $media_array = $this->scrape_instagram( $username );

      if ( is_wp_error( $media_array ) ) {
        printf( '<div class="error-message">%s</div>', wp_kses_post( $media_array->get_error_message() ) );
      } else {

        // filter for images only?
        if ( $images_only = apply_filters( 'wdb_images_only', false ) ) {
          $media_array = array_filter( $media_array, array( $this, 'images_only' ) );
        }

        // slice list down to required limit.
        $media_array = array_slice( $media_array, 0, $limit );
        ?>

        <ul class="instagram-pics clearfix">
          <?php
          foreach ( $media_array as $item ) {
            printf( '
              <li>
                <a href="%1$s" target="_blank" rel="nofollow">
                <img src="%2$s" alt="%3$s" title="%3$s" />
                </a>
              </li>', 
              esc_url( $item['link'] ), 
              esc_url( $item[$size] ), 
              esc_attr( $item['description'] )
            );
          }
          ?>
        </ul>
        
        <?php
      }
    }

    switch ( substr( $username, 0, 1 ) ) {
      case '#':
        $url = '//instagram.com/explore/tags/' . str_replace( '#', '', $username );
        break;
      default:
        $url = '//instagram.com/' . str_replace( '@', '', $username );
        break;
    }

    if ( '' !== $link ) {
      ?>

      <p><a href="<?php echo trailingslashit( esc_url( $url ) ); ?>" rel="me" target="_blank""><?php echo esc_html( $link ); ?></a></p>

      <?php
    }

    echo $args['after_widget'];
  }

  function form( $instance ) {
    $instance = wp_parse_args( (array) $instance, 
      array( 
        'title'    => esc_html__( 'Instagram', 'wildbook' ), 
        'username' => '', 
        'link'     => esc_html__( 'Follow Us', 'wildbook' ), 
        'size'     => 'thumbnail',
        'number'   => 9
      ) 
    );
  ?>
    <p>
      <label for="<?php echo esc_attr( $this->get_field_id('title') ); ?>"><?php esc_html_e( 'Title:', 'wildbook' ); ?>
        <input class="widefat" id="<?php echo esc_attr( $this->get_field_id('title') ); ?>" name="<?php echo esc_attr( $this->get_field_name('title') ); ?>" type="text" value="<?php echo esc_attr( $instance['title'] ); ?>" />
      </label>
    </p>
    
    <p>
      <label for="<?php echo esc_attr( $this->get_field_id('username') ); ?>"><?php esc_html_e( 'Username:', 'wildbook' ); ?> 
        <input class="widefat" id="<?php echo esc_attr( $this->get_field_id('username') ); ?>" name="<?php echo esc_attr( $this->get_field_name('username') ); ?>" type="text" value="<?php echo esc_attr( $instance['username'] ); ?>" />
      </label>
    </p>
    
    <p>
      <label for="<?php echo esc_attr( $this->get_field_id('number') ); ?>"><?php esc_html_e( 'Number of photos:', 'wildbook' ); ?> 
        <input class="widefat" id="<?php echo esc_attr( $this->get_field_id('number') ); ?>" name="<?php echo esc_attr( $this->get_field_name('number') ); ?>" type="text" value="<?php echo absint( $instance['number'] ); ?>" />
      </label>
    </p>
    
    <p>
      <label for="<?php echo esc_attr( $this->get_field_id( 'size' ) ); ?>"><?php esc_html_e( 'Photo size', 'wildbook' ); ?>:</label>
      <select id="<?php echo esc_attr( $this->get_field_id( 'size' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'size' ) ); ?>" class="widefat">
        <option value="thumbnail" <?php selected( 'thumbnail', $instance['size'] ) ?>><?php esc_html_e( 'Thumbnail', 'wildbook' ); ?></option>
        <option value="small" <?php selected( 'small', $instance['size'] ) ?>><?php esc_html_e( 'Small', 'wildbook' ); ?></option>
        <option value="large" <?php selected( 'large', $instance['size'] ) ?>><?php esc_html_e( 'Large', 'wildbook' ); ?></option>
        <option value="original" <?php selected( 'original', $instance['size'] ) ?>><?php esc_html_e( 'Original', 'wildbook' ); ?></option>
      </select>
    </p>
    
    <p>
      <label for="<?php echo esc_attr( $this->get_field_id('link') ); ?>"><?php esc_html_e( 'Link text:', 'wildbook' ); ?> 
        <input class="widefat" id="<?php echo esc_attr( $this->get_field_id('link') ); ?>" name="<?php echo esc_attr( $this->get_field_name('link') ); ?>" type="text" value="<?php echo esc_attr( $instance['link'] ); ?>" />
      </label>
    </p>
    <?php
  }

  function update( $new_instance, $old_instance ) {
    $instance = $old_instance;
    $instance['title']    = strip_tags( $new_instance['title'] );
    $instance['username'] = trim( strip_tags( $new_instance['username'] ) );
    $instance['number']   = ! absint( $new_instance['number'] ) ? 9 : $new_instance['number'];
    $instance['size'] = ( ( $new_instance['size'] == 'thumbnail' || $new_instance['size'] == 'large' || $new_instance['size'] == 'small' || $new_instance['size'] == 'original' ) ? $new_instance['size'] : 'large' );
    $instance['link']     = strip_tags( $new_instance['link'] );
    return $instance;
  }

  // based on https://gist.github.com/cosmocatalano/4544576.
  // https://github.com/scottsweb/wp-instagram-widget
  function scrape_instagram( $username ) {

    $username = trim( strtolower( $username ) );

      switch ( substr( $username, 0, 1 ) ) {
        case '#':
          $url              = 'https://instagram.com/explore/tags/' . str_replace( '#', '', $username );
          $transient_prefix = 'h';
          break;

        default:
          $url              = 'https://instagram.com/' . str_replace( '@', '', $username );
          $transient_prefix = 'u';
          break;
      }

      if ( false === ( $instagram = get_transient( 'insta-a10-' . $transient_prefix . '-' . sanitize_title_with_dashes( $username ) ) ) ) {

        $remote = wp_remote_get( $url );

        if ( is_wp_error( $remote ) ) {
          return new WP_Error( 'site_down', esc_html__( 'Unable to communicate with Instagram.', 'wildbook' ) );
        }

        if ( 200 !== wp_remote_retrieve_response_code( $remote ) ) {
          return new WP_Error( 'invalid_response', esc_html__( 'Instagram did not return a 200.', 'wildbook' ) );
        }

        $shards      = explode( 'window._sharedData = ', $remote['body'] );
        $insta_json  = explode( ';</script>', $shards[1] );
        $insta_array = json_decode( $insta_json[0], true );

        if ( ! $insta_array ) {
          return new WP_Error( 'bad_json', esc_html__( 'Instagram has returned invalid data.', 'wildbook' ) );
        }

        if ( isset( $insta_array['entry_data']['ProfilePage'][0]['graphql']['user']['edge_owner_to_timeline_media']['edges'] ) ) {
          $images = $insta_array['entry_data']['ProfilePage'][0]['graphql']['user']['edge_owner_to_timeline_media']['edges'];
        } elseif ( isset( $insta_array['entry_data']['TagPage'][0]['graphql']['hashtag']['edge_hashtag_to_media']['edges'] ) ) {
          $images = $insta_array['entry_data']['TagPage'][0]['graphql']['hashtag']['edge_hashtag_to_media']['edges'];
        } else {
          return new WP_Error( 'bad_json_2', esc_html__( 'Instagram has returned invalid data.', 'wildbook' ) );
        }

        if ( ! is_array( $images ) ) {
          return new WP_Error( 'bad_array', esc_html__( 'Instagram has returned invalid data.', 'wildbook' ) );
        }

        $instagram = array();

        foreach ( $images as $image ) {
          if ( true === $image['node']['is_video'] ) {
            $type = 'video';
          } else {
            $type = 'image';
          }

          $caption = __( 'Instagram Image', 'wildbook' );
          if ( ! empty( $image['node']['edge_media_to_caption']['edges'][0]['node']['text'] ) ) {
            $caption = wp_kses( $image['node']['edge_media_to_caption']['edges'][0]['node']['text'], array() );
          }

          $instagram[] = array(
            'description' => $caption,
            'link'        => trailingslashit( '//instagram.com/p/' . $image['node']['shortcode'] ),
            'time'        => $image['node']['taken_at_timestamp'],
            'comments'    => $image['node']['edge_media_to_comment']['count'],
            'likes'       => $image['node']['edge_liked_by']['count'],
            'thumbnail'   => preg_replace( '/^https?\:/i', '', $image['node']['thumbnail_resources'][0]['src'] ),
            'small'       => preg_replace( '/^https?\:/i', '', $image['node']['thumbnail_resources'][2]['src'] ),
            'large'       => preg_replace( '/^https?\:/i', '', $image['node']['thumbnail_resources'][4]['src'] ),
            'original'    => preg_replace( '/^https?\:/i', '', $image['node']['display_url'] ),
            'type'        => $type,
          );
        } // End foreach().

        // do not set an empty transient - should help catch private or empty accounts.
        if ( ! empty( $instagram ) ) {
          $instagram = serialize( $instagram );
          set_transient( 'insta-a10-' . $transient_prefix . '-' . sanitize_title_with_dashes( $username ), $instagram, apply_filters( 'wdb_instagram_cache_time', HOUR_IN_SECONDS * 2 ) );
        }
      }

      if ( ! empty( $instagram ) ) {
        return unserialize( $instagram );
      } else {
        return new WP_Error( 'no_images', esc_html__( 'Instagram did not return any images.', 'wildbook' ) );
      }
  }

  function images_only( $media_item ) {
    if ( 'image' === $media_item['type'] ) {
        return true;
    }

    return false;
  }
}

/**
 * Recent Posts with Thumbnails
 *
 * @since Wild Book 1.4.7
 */
class Wild_Book_Recent_Posts extends WP_Widget {

  private $orderby;
  
  /**
   * Constructor.
   */
  public function __construct() {
  
    parent::__construct( 'wildbook-widget-recent-posts', esc_html__( 'Recents Posts with Thumbnails', 'wildbook' ), array(
      'classname'   => 'wildbook-widget-recent-posts',
      'description' => esc_html__( 'Use this widget to list your recent posts.', 'wildbook' ),
    ) );

    $this->orderby = array(
     'date'         => esc_html__( 'date', 'wildbook' ), 
     'comments_num' => esc_html__( 'number of comments', 'wildbook' ),
     'random'       => esc_html__( 'random', 'wildbook' )
    );
    
    add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );
  }
  
    /**
   * Enqueue admin widgets page scripts and styles.
   *
   * @access public
   */ 
  public function admin_enqueue_scripts( $hook ) {
    if ( 'widgets.php' != $hook )
          return;

    wp_enqueue_style( 'wildbook-widget-recent-posts-css', get_template_directory_uri() . '/css/admin/widget-recent-posts.css', array(), '1.0'); 
  }
  
  /**
   * Output the HTML for this widget.
   *
   * @access public
   *
   * @param array $args     An array of standard parameters for widgets in this theme.
   * @param array $instance An array of settings for this widget instance.
   */
  public function widget( $args, $instance ) {
  
    $number = ( ! empty( $instance['number'] ) ) ? absint( $instance['number'] ) : 5;
    $title = ( ! empty( $instance['title'] ) ) ? strip_tags( $instance['title'] ) : esc_html__( 'Recent Posts', 'wildbook' );
  
    $query_args = array(
      'order'          => 'DESC',
      'posts_per_page' => $number,
      'no_found_rows'  => true,
      'post_type'      => 'post',
      'post_status'    => 'publish',
      'post__not_in'   => get_option( 'sticky_posts' )
    );
    
    $query_args['tax_query'] = array();
    
    if( ! empty( $instance['category'] ) ){
      $query_args['tax_query'][] = array(
          'taxonomy' => 'category',
          'terms'    => absint( $instance['category'] ),
          'field'    => 'term_id'
      );
    }
    
    if( ! empty( $instance['tags'] ) ) {
      $tags = explode( ',', $instance['tags'] );
      $query_args['tax_query'][] = array(
          'taxonomy' => 'post_tag',
          'terms'    => $tags,
          'field'    => 'name'
      );
    }

    if( count( $query_args['tax_query'] ) > 1 ){
      $query_args['tax_query']['relation'] = 'AND';
    }

    $orderby = isset( $instance['orderby'] ) ? $instance['orderby'] : 'date';
    
    switch ($orderby) {
      case 'comments_num':
        $query_args['orderby'] = 'comment_count';
        break;
      case 'random':
        $query_args['orderby'] = 'rand';
        break;
    }

    $recent_posts = new WP_Query( $query_args );

    if ( $recent_posts->have_posts() ) :
      $show_date = isset( $instance['show_date'] ) ? $instance['show_date'] : false;

      echo $args['before_widget'];
      
      if ( ! empty( $title ) ) {
        printf( '%s%s%s', $args['before_title'], $title, $args['after_title'] );
      } 
    ?>
      
    <ul>
      <?php while ( $recent_posts->have_posts() ) : $recent_posts->the_post(); ?>
      
      <li class="widget-post clearfix <?php printf( 'format-%s', get_post_format() ? get_post_format() : 'standard' ); ?><?php if ( has_post_thumbnail() ) : ?> has-post-thumbnail<?php endif; ?>">
      
        <?php if ( has_post_thumbnail() ) : ?>
        <div class="post-thumbnail">
          <a href="<?php the_permalink(); ?>"><?php the_post_thumbnail( 'thumbnail' ); ?></a>
        </div>
        <?php endif; ?>
        
        <header class="widget-post-content">
          <?php 
            if( ! get_the_title() ) {
              $format = get_post_format();
              $default_title = ( ! $format || $format == 'standard' ) ? esc_html__( 'Untitled Post', 'wildbook' ) : get_post_format_string( $format );
            }
          ?>
          <a href="<?php the_permalink(); ?>"><?php get_the_title() ? the_title() : printf( $default_title ); ?></a>
          <?php 
            if( $show_date ) {
              printf( '<span class="post-date">%s</span>', get_the_date() );
            } 
          ?>
        </header>
      </li>  
      <?php endwhile; ?>
    </ul> 
      
    <?php
      echo $args['after_widget'];

      // Reset the post globals as this query will have stomped on it.
      wp_reset_postdata();
    endif;
  }
  
  /**
   * Deal with the settings when they are saved by the admin.
   *
   * Here is where any validation should happen.
   *
   * @param array $new_instance New widget instance.
   * @param array $instance     Original widget instance.
   * @return array Updated widget instance.
   */
  function update( $new_instance, $instance ) {
    $instance['title']  = strip_tags( $new_instance['title'] );
    $instance['number'] = ( !empty( $new_instance['number'] ) ) ? absint( $new_instance['number'] ) : 5;
    
    if( array_key_exists( $new_instance['orderby'], $this->orderby ) ){
      $instance['orderby'] = $new_instance['orderby'];
    }

    $instance['category']   = isset( $new_instance['category'] ) ? absint( $new_instance['category'] ) : 0;
    $instance['tags']       = strip_tags( $new_instance['tags'] );
    $instance['show_date']  = isset( $new_instance['show_date'] ) ? (bool) $new_instance['show_date'] : false;
    return $instance;
  }
  
  /**
   * Display the form for this widget on the Widgets page of the Admin area.
   *
   * @param array $instance
   */
  function form( $instance ) {
    $instance = wp_parse_args( (array) $instance, array( 
      'title'      => '', 
      'number'     => 5, 
      'orderby'    => 'date',
      'category'   => 0,
      'tags'       => '',
      'show_date'  => true
    ) );
  ?>
    
    <p><label for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"><?php esc_html_e( 'Title:', 'wildbook' ); ?></label>
    <input id="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>" class="widefat" name="<?php echo esc_attr( $this->get_field_name( 'title' ) ); ?>" type="text" value="<?php echo esc_attr( $instance['title'] ); ?>"></p>

    <p><label for="<?php echo esc_attr( $this->get_field_id( 'number' ) ); ?>"><?php esc_html_e( 'Number of posts to show:', 'wildbook' ); ?></label>
    <input id="<?php echo esc_attr( $this->get_field_id( 'number' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'number' ) ); ?>" type="text" value="<?php echo absint( $instance['number'] ); ?>" size="3"></p>

    <p>
      <label for="<?php echo esc_attr( $this->get_field_id( 'orderby' ) ); ?>"><?php esc_html_e( 'Order by:', 'wildbook' ); ?></label>
      <select id="<?php echo esc_attr( $this->get_field_id( 'orderby' ) ); ?>" class="widefat" name="<?php echo esc_attr( $this->get_field_name( 'orderby' ) ); ?>">
        <?php foreach ( $this->orderby as $slug => $name ) : ?>
        <option value="<?php echo esc_attr( $slug ); ?>"<?php selected( $instance['orderby'], $slug ); ?>><?php echo esc_attr( $name ); ?></option>
        <?php endforeach; ?>
      </select>
    </p>
    
    <p>
      <label for="<?php echo esc_attr( $this->get_field_id( 'category' ) ); ?>"><?php esc_html_e( 'Category:', 'wildbook' ); ?></label>
      <?php
        wp_dropdown_categories( array(
          'show_option_all' => esc_html__( 'All Categories', 'wildbook' ),
          'selected'        => absint( $instance['category'] ),
          'name'            => esc_attr( $this->get_field_name( 'category' ) ),
          'id'              => esc_attr( $this->get_field_id( 'category' ) ),
          'class'           => 'widefat'
        ) );
      ?>
    </p>
    
    <p><label for="<?php echo esc_attr( $this->get_field_id( 'tags' ) ); ?>"><?php esc_html_e( 'Tags (separated by commas):', 'wildbook' ); ?></label>
    <input id="<?php echo esc_attr( $this->get_field_id( 'tags' ) ); ?>" class="widefat" name="<?php echo esc_attr( $this->get_field_name( 'tags' ) ); ?>" type="text" value="<?php echo esc_attr( $instance['tags'] ); ?>"></p>
    
    <p><input class="checkbox" type="checkbox" <?php checked( $instance['show_date'] ); ?> id="<?php echo esc_attr( $this->get_field_id( 'show_date' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'show_date' ) ); ?>" />
    <label for="<?php echo $this->get_field_id( 'show_date' ); ?>"><?php esc_html_e( 'Display post date?', 'wildbook' ); ?></label></p>
    
  <?php
  }
}
?>