/**
 * Theme functions file.
 */
(function($) {
  "use strict";

  $(document).ready(function($){

    var $body = $('body');

    if( ! $body.hasClass( 'preloader-off' ) ) {
      $($body).waitForImages(function() {
        $body.addClass('loaded');
      }); 
    }
    
    if( $body.hasClass( 'sticky-nav' ) && $( '#top-nav').length ) {
      $('.site-content').css( 'padding-top', $( '#top-nav').outerHeight() );
    }
    
    // Sidebar Nav Toggle
    $('.nav-toggle').on( 'click', function(e) {
      e.preventDefault();
      $body.toggleClass('nav-open');
    });

    $('#close-canvas-nav').on( 'click', function(e) {
      e.preventDefault();
      $body.removeClass('nav-open');
    });

    $('.search-toggle').on( 'click', function(e) {
      e.preventDefault();
      var $searchForm = $(this).next('.search-form'),
          $searchField = $searchForm.find('.search-field');
      if( $searchField.val() ) {
        $searchForm.submit();
      } else {
        $body.toggleClass('search-open');
      }
    });

    // Sidebar Nav
    var $navMenu = $('.sidebar-nav .nav-menu');

    if( $navMenu.length ) {
      $navMenu.children('li').addClass('menu-item-parent');
      
      $navMenu.find('.menu-item-has-children > a').on('click', function(e){
        e.preventDefault();
        var itemSub = $(this).next('.sub-menu'),
            parentSubs = $(this).closest('.menu-item-parent').find('.sub-menu');
        
        $navMenu.find('.sub-menu').not(parentSubs).slideUp(250);
        itemSub.slideToggle(250);
      }); 
    }
    
    // Top Navigation
    $('#top-nav-toggle').on('click', function(e) {
      e.preventDefault();
      var $btn = $(this),
          $btnSpan = $btn.children('span');
      
      $btn.next().slideToggle(500);
      
      if( $btnSpan.text() == $btn.attr('data-close-text') ) {
        $btnSpan.text($btn.attr('title'));
      } else {
        $btnSpan.text($btn.attr('data-close-text'));
      }
    });

    var $masonry_container = $( '.masonry-container' );

    if( $.fn.masonry !== undefined && $masonry_container.length ) {
      $masonry_container.imagesLoaded(function() {
        $masonry_container.masonry({
          itemSelector: '.gallery-item',
          isRTL: $body.hasClass( 'rtl' ) ? true : false
        });
      });
    }

    var $owl_container = $( '.owl-carousel' );

    if( $.fn.owlCarousel !== undefined && $owl_container.length ) {

      $owl_container.each( function() {
        var $owl = $(this),
            columns = $owl.attr( 'data-columns' );

        $owl.waitForImages(function() {
          $owl.owlCarousel({
            loop: true,
            margin: 10, 
            autoHeight: true, 
            smartSpeed: columns < 2 ? 600 : 300,
            dots: columns < 2 ? false : true,
            nav: columns < 2 ? true : false,
            rtl: $body.hasClass( 'rtl' ) ? true : false,
            navText : [ $body.hasClass( 'rtl' ) ? '<i class="fa fa-angle-right"></i>' : '<i class="fa fa-angle-left"></i>', $body.hasClass( 'rtl' ) ? '<i class="fa fa-angle-left"></i>' : '<i class="fa fa-angle-right"></i>' ],
            responsive:{
                0:{
                  items: columns < 2 ? parseInt( columns ) : 2,
                  margin: 5
                },
                768:{
                  items: parseInt( columns )
                }
            }
          });
        });
      });
    }
    
    // Share Buttons
    $('.share-button').on( 'click', function(e) {
      e.preventDefault();
      var $button = $(this),
          newWindowWidth = 550,
          newWindowHeight = 450,
          leftPos = ( $( window ).width()/2 ) - ( newWindowWidth/2 ),
          topPos = ( $( window ).height()/2 ) - ( newWindowHeight/2 );
          
			var newWindow = window.open( $button.attr('href'), '', 'height=' + newWindowWidth + ', width=' + newWindowHeight + ', left=' + leftPos + ', top=' + topPos );

			if ( window.focus )
				newWindow.focus();
    });
    
    // Fitvids init
    $('.entry-content').fitVids();
    
    if( $body.hasClass( 'snow-on' ) ) {
      $('#sidebar').snowfall( { flakeCount : 150, minSpeed : 2 });
    }
    
  });
})(jQuery);